from . import Image
import numpy as np

import pydicom
from pydicom.encaps import encapsulate
from pydicom.uid import JPEG2000, RLELossless, ImplicitVRLittleEndian




class ImageIO:

    # private
    def ensure_even(self, stream):
        # Very important for some viewers
        if len(stream) % 2:
            return stream + b"\x00"
        return stream

    def loadImage(self, file):
        ds = pydicom.dcmread(file)
        pixel_array = np.uint16(ds.pixel_array)
        cols = ds.Columns
        rows = ds.Rows
        return Image(pixel_array, cols, rows, ds)

    def buildFile(self, image, saveName):
        # TODO : edit height and width elements
        dataset = ImageIO.buildDataSetImplicitVRLittleEndian(image)
        dataset.save_as(saveName)
        return saveName

    def buildDataSetJPEG2000(self, image):
        from io import BytesIO
        from PIL import Image as PImage
        dataset = image.dataset
        pixels = image.pixelData
        frame_data = []
        with BytesIO() as output:
            image = PImage.fromarray(pixels)
            image.save(output, format="JPEG2000")
            frame_data.append(output.getvalue())
        dataset.PixelData = encapsulate(frame_data)
        dataset.file_meta.TransferSyntaxUID = JPEG2000
        dataset.is_implicit_VR = False
        return dataset

    def buildDataSetImplicitVRLittleEndian(self, image):
        dataset = image.dataset
        pixels = image.pixelData
        dataset.PixelData = pixels.tobytes()
        dataset.file_meta.TransferSyntaxUID = ImplicitVRLittleEndian
        dataset.is_implicit_VR = True
        return dataset



import logging

"""
this a small example of a noop
"""
class GenericOperation:

    def init(self, params):
        """if you have any sort of predefined varibles or  parameters tha need to be initialize

        Args:
            params : intilization parameters
        """
        logging.info('Init')  # will not print anything

    def applyTransformation(self, image, params):
        """
        Main Entry Point for operations 
        image : Image Container  
        image.pixelData = numpy array  
        image.height = cols  
        image.width = rows  
        image.dataset = dataset ( original image file  container refrence)   

        #### example : https://pydicom.github.io/pydicom/stable/auto_examples/image_processing/plot_downsize_image.html#sphx-glr-auto-examples-image-processing-plot-downsize-image-py
        ```
            get the pixel information into a numpy array
            data = ds.pixel_array
            data_downsampling = data[::8, ::8]
            data_downsampling.shape[0], data_downsampling.shape[1]))
        ```
        """
        return image



from setuptools import setup, find_packages
from glob import glob

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('README.md') as readme_file:
    README = readme_file.read()

with open('HISTORY.md') as history_file:
    HISTORY = history_file.read()


setup_args = dict(
    name="latis-image-proccessing",  # Replace with your own username
    version="0.0.7",
    author="Mohamed Amine",
    author_email="hiddentn@outlook.com",
    description="latis image proccessing utilities for python",
    # long_description=README + '\n\n' + HISTORY,
    # long_description_content_type="text/markdown",
    url="http://www.latis-eniso.org/",
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Microsoft :: Windows"],
    python_requires='==3.7',
    packages=find_packages(),
    data_files=[('lib\\site-packages\\gdsm', glob("gdsm/bin/*"))],
)
install_requires = [
    'pydicom',
    'numpy',
    'Pillow'
]

if __name__ == '__main__':
    setup(**setup_args, install_requires=install_requires,
          include_package_data=True)
